"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Point_1 = require("./Point");
const Solver_1 = require("./Solver");
const Vector3_1 = require("./Vector3");
class Field {
    constructor(points, solver = new Solver_1.Solver(), frame = points[0], withMass = false) {
        this.points = points;
        this.frame = frame;
        this._setSolvers(solver, withMass);
    }
    get solver() {
        return this.points[0].solver;
    }
    set solver(newSolver) {
        this._setSolvers(newSolver, false);
    }
    get barycenter() {
        let mass = this.points.reduce((acc, point) => acc + point.mass, 0);
        return this.points.reduce((acc, point) => acc.add(point.position.mul(point.mass)), Vector3_1.Vector3.zeros).div(mass);
    }
    set barycenter(newCenter) {
        let center = this.barycenter;
        this.points.forEach((point) => {
            point.position = point.position.sub(center).add(newCenter);
        });
    }
    _setSolvers(solver, withMass) {
        this.points.forEach((point) => {
            let f = solver.f;
            point.solver = withMass ? new Solver_1.Solver((u) => f(u).div(point.mass), solver.dt0, solver.method) : solver;
        });
    }
    update(dt, origin) {
        this.points = this.points.map((point) => point.copy().update(dt, origin));
        return this;
    }
    reframe(p) {
        p = p || Point_1.Point.zeros();
        this.points.forEach((q) => {
            q.reframe(p);
        });
        return this;
    }
}
exports.Field = Field;
//# sourceMappingURL=Field.js.map