"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Vector3_1 = require("./Vector3");
class Matrix3 {
    constructor(xx = 0, xy = 0, xz = 0, yx = 0, yy = 0, yz = 0, zx = 0, zy = 0, zz = 0) {
        this.x = new Vector3_1.Vector3(xx, xy, xz);
        this.y = new Vector3_1.Vector3(yx, yy, yz);
        this.z = new Vector3_1.Vector3(zx, zy, zz);
    }
    get norm() {
        return Math.sqrt(this.scal(this));
    }
    row(i) {
        return i === 0 ? this.x : undefined || i === 1 ? this.y : undefined || i === 2 ? this.z : undefined;
    }
    col(j) {
        return new Vector3_1.Vector3(this.x.to1D()[j], this.y.to1D()[j], this.z.to1D()[j]);
    }
    set(xx = 0, xy = 0, xz = 0, yx = 0, yy = 0, yz = 0, zx = 0, zy = 0, zz = 0) {
        this.x.xyz = [xx, xy, xz];
        this.y.xyz = [yx, yy, yz];
        this.z.xyz = [zx, zy, zz];
        return this;
    }
    copy() {
        return new Matrix3(...this.to1D());
    }
    fill(s) {
        this.x.fill(s);
        this.y.fill(s);
        this.z.fill(s);
        return this;
    }
    fillc(s) {
        return this.copy().fill(s);
    }
    add(m) {
        this.x.add(m.x);
        this.y.add(m.y);
        this.z.add(m.z);
        return this;
    }
    addc(m) {
        return this.copy().add(m);
    }
    sub(m) {
        this.x.sub(m.x);
        this.y.sub(m.y);
        this.z.sub(m.z);
        return this;
    }
    subc(m) {
        return this.copy().sub(m);
    }
    opp() {
        this.x.opp();
        this.y.opp();
        this.z.opp();
        return this;
    }
    oppc() {
        return this.copy().opp();
    }
    mul(s) {
        this.x.mul(s);
        this.y.mul(s);
        this.z.mul(s);
        return this;
    }
    mulc(s) {
        return this.copy().mul(s);
    }
    div(s) {
        this.x.div(s);
        this.y.div(s);
        this.z.div(s);
        return this;
    }
    divc(s) {
        return this.copy().div(s);
    }
    trans() {
        this.set(this.x.x, this.y.x, this.z.x, this.x.y, this.y.y, this.z.y, this.x.z, this.y.z, this.z.z);
        return this;
    }
    transc() {
        return this.copy().trans();
    }
    prod(m) {
        const mTrs = m.copy().trans();
        this.set(this.x.scal(mTrs.x), this.x.scal(mTrs.y), this.x.scal(mTrs.z), this.y.scal(mTrs.x), this.y.scal(mTrs.y), this.y.scal(mTrs.z), this.z.scal(mTrs.x), this.z.scal(mTrs.y), this.z.scal(mTrs.z));
        return this;
    }
    prodc(m) {
        return this.copy().prod(m);
    }
    scal(m) {
        return this.x.scal(m.x) + this.y.scal(m.y) + this.z.scal(m.z);
    }
    dist(m) {
        return this.copy().sub(m).norm;
    }
    map(u) {
        u.xyz = [this.x.scal(u), this.y.scal(u), this.z.scal(u)];
        return u;
    }
    mapc(u) {
        return this.map(u.copy());
    }
    get det() {
        return this.x.x * this.y.y * this.z.z + this.x.y * this.y.z * this.z.x + this.x.z * this.y.x * this.z.y
            - this.x.z * this.y.y * this.z.x - this.x.y * this.y.x * this.z.z - this.x.x * this.y.z * this.z.y;
    }
    inv() {
        const det = this.det;
        this.set(this.y.y * this.z.z - this.y.z * this.z.y, this.x.z * this.z.y - this.x.y * this.z.z, this.x.y * this.y.z - this.x.z * this.y.y, this.y.z * this.z.x - this.y.x * this.z.z, this.x.x * this.z.z - this.x.z * this.z.x, this.x.z * this.y.x - this.x.x * this.y.z, this.y.x * this.z.y - this.y.y * this.z.x, this.x.y * this.z.x - this.x.x * this.z.y, this.x.x * this.y.y - this.x.y * this.y.x).div(det);
        return this;
    }
    invc() {
        return this.copy().inv();
    }
    rpow(exp) {
        if (exp > 1) {
            const copy = this.copy();
            this.prod(copy);
            if (exp % 2 === 0) {
                this.rpow(exp / 2);
            }
            else if (exp % 2 === 1) {
                this.rpow((exp - 1) / 2);
                this.prod(copy);
            }
        }
    }
    pow(exp) {
        if (exp < 0)
            this.inv();
        if (exp === 0)
            this.set(...Matrix3.eye.to1D());
        this.rpow(Math.abs(exp));
        return this;
    }
    powc(exp) {
        return this.copy().pow(exp);
    }
    isEqual(m) {
        return this.x.isEqual(m.x) && this.y.isEqual(m.y) && this.z.isEqual(m.z);
    }
    isZero() {
        return this.x.isZero() && this.y.isZero() && this.z.isZero();
    }
    toString() {
        return `${this.x.toString()}\n${this.y.toString()}\n${this.z.toString()}`;
    }
    to2D() {
        return [this.x.to1D(), this.y.to1D(), this.z.to1D()];
    }
    to1D() {
        return [...this.x.to1D(), ...this.y.to1D(), ...this.z.to1D()];
    }
    static get zeros() {
        return new Matrix3();
    }
    static get ones() {
        return new Matrix3(1, 1, 1, 1, 1, 1, 1, 1, 1);
    }
    static get eye() {
        return new Matrix3(1, 0, 0, 0, 1, 0, 0, 0, 1);
    }
    static scal(s) {
        return new Matrix3(s, 0, 0, 0, s, 0, 0, 0, s);
    }
    static diag(xx, yy, zz) {
        return new Matrix3(xx, 0, 0, 0, yy, 0, 0, 0, zz);
    }
    static sym(xx, yy, zz, xy, yz, xz = 0) {
        return new Matrix3(xx, xy, xz, xy, yy, yz, xz, yz, zz);
    }
    static asym(xy, yz, xz = 0) {
        return new Matrix3(0, xy, xz, -xy, 0, yz, -xz, -yz, 0);
    }
    static e(i, j) {
        const row = new Vector3_1.Vector3(j === 0 ? 1 : 0, j === 1 ? 1 : 0, j === 2 ? 1 : 0);
        const can = Matrix3.zeros;
        can.x = i == 0 ? row : can.x;
        can.y = i == 1 ? row : can.y;
        can.z = i == 2 ? row : can.z;
        return can;
    }
    static rotX(theta, cos = Math.cos, sin = Math.sin) {
        return new Matrix3(1, 0, 0, 0, cos(theta), -sin(theta), 0, sin(theta), cos(theta));
    }
    static rotY(theta, cos = Math.cos, sin = Math.sin) {
        return new Matrix3(cos(theta), 0, sin(theta), 0, 1, 0, -sin(theta), 0, cos(theta));
    }
    static rotZ(theta, cos = Math.cos, sin = Math.sin) {
        return new Matrix3(cos(theta), -sin(theta), 0, sin(theta), cos(theta), 0, 0, 0, 1);
    }
    static makeRot(u, cos = Math.cos, sin = Math.sin) {
        return (theta) => {
            const k = 1 - cos(theta), c = cos(theta), s = sin(theta);
            return new Matrix3(k * Math.pow(u.x, 2) + c, k * u.x * u.y - u.z * s, k * u.x * u.z + u.y * s, k * u.x * u.y + u.z * s, k * Math.pow(u.y, 2) + c, k * u.y * u.z - u.x * s, k * u.x * u.z - u.y * s, k * u.y * u.z + u.x * s, k * Math.pow(u.z, 2) + c);
        };
    }
    static makeAffine(m, v) {
        return (u) => m.map(u).add(v);
    }
    static tens(u, v) {
        const right = v || u;
        return new Matrix3(u.x * right.x, u.x * right.y, u.x * right.z, u.y * right.x, u.y * right.y, u.y * right.z, u.z * right.x, u.z * right.y, u.z * right.z);
    }
    static from2D(arr) {
        return new Matrix3(arr[0][0], arr[0][1], arr[0][2], arr[1][0], arr[1][1], arr[1][2], arr[2][0], arr[2][1], arr[2][2]);
    }
    static from1D(arr) {
        return new Matrix3(arr[0], arr[1], arr[2], arr[3], arr[4], arr[5], arr[6], arr[7], arr[8]);
    }
}
exports.Matrix3 = Matrix3;
//# sourceMappingURL=Matrix3.js.map