"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Vector3_1 = require("./Vector3");
const Matrix3_1 = require("./Matrix3");
class Pair3 {
    constructor(origin = Vector3_1.Vector3.zeros, position = Vector3_1.Vector3.zeros) {
        this.origin = origin.copy();
        this.position = position.copy();
    }
    get length() {
        return this.position.dist(this.origin);
    }
    set length(len) {
        this.relative = this.length > 0 ? this.relative.mul(len / this.length) : Vector3_1.Vector3.zeros;
    }
    get relative() {
        return this.position.subc(this.origin);
    }
    set relative(rel) {
        this.position = rel.addc(this.origin);
    }
    translate(u) {
        this.origin.add(u);
        this.position.add(u);
        return this;
    }
    homothetic(s) {
        this.origin.mul(s);
        this.position.mul(s);
        return this;
    }
    transform(m) {
        m.map(this.origin);
        m.map(this.position);
        return this;
    }
    affine(m, v) {
        const aff = Matrix3_1.Matrix3.makeAffine(m, v);
        this.origin = aff(this.origin);
        this.position = aff(this.position);
        return this;
    }
    copy() {
        return new Pair3(this.origin, this.position);
    }
    isEqual(om) {
        return this.relative.isEqual(om.relative);
    }
    isZero() {
        return this.origin.isEqual(this.position);
    }
    toString() {
        return `origin ${this.origin.toString()}\t vector ${this.position.toString()}`;
    }
    static zeros(u = Vector3_1.Vector3.zeros) {
        return new Pair3(u, u);
    }
    static vect(u) {
        return new Pair3(Vector3_1.Vector3.zeros, u);
    }
}
exports.Pair3 = Pair3;
//# sourceMappingURL=Pair3.js.map