"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Vector3_1 = require("./Vector3");
const Solver_1 = require("./Solver");
const BufferTrajectory_1 = require("./BufferTrajectory");
class Point {
    constructor(trajectory, solver = new Solver_1.Solver(), mass = 0) {
        this.trajectory = trajectory;
        this.solver = solver;
        this.mass = mass;
    }
    get position() {
        return this.trajectory.last.relative;
    }
    set position(newPosition) {
        this.trajectory.last.relative = newPosition;
    }
    get speed() {
        return this.du.div(this.dt);
    }
    set speed(newSpeed) {
        this.init(this.position, newSpeed);
    }
    get du() {
        return this.trajectory.last.relative.sub(this.trajectory.nexto.relative);
    }
    set du(newDu) {
        this.trajectory.last.relative = this.trajectory.nexto.relative.add(newDu);
    }
    get dt() {
        return this.trajectory.dt[this.trajectory.lastStepIndex];
    }
    get x() {
        return this.trajectory.last.relative.x;
    }
    set x(newX) {
        this.trajectory.last.position.x = newX + this.trajectory.last.origin.x;
    }
    get y() {
        return this.trajectory.last.relative.y;
    }
    set y(newY) {
        this.trajectory.last.position.y = newY + this.trajectory.last.origin.y;
    }
    get z() {
        return this.trajectory.last.relative.z;
    }
    set z(newZ) {
        this.trajectory.last.position.z = newZ + this.trajectory.last.origin.z;
    }
    get r() {
        return this.trajectory.last.relative.r;
    }
    set r(newR) {
        this.trajectory.last.position.r = newR + this.trajectory.last.origin.r;
    }
    get rxy() {
        return this.trajectory.last.relative.rxy;
    }
    set rxy(newRxy) {
        this.trajectory.last.position.rxy = newRxy + this.trajectory.last.origin.rxy;
    }
    get theta() {
        return this.trajectory.last.relative.theta;
    }
    set theta(newTheta) {
        let relative = this.trajectory.last.relative;
        relative.theta = newTheta;
        this.trajectory.last.relative = relative;
    }
    get phi() {
        return this.trajectory.last.relative.phi;
    }
    set phi(newPhi) {
        let relative = this.trajectory.last.relative;
        relative.phi = newPhi;
        this.trajectory.last.relative = relative;
    }
    get lat() {
        return this.trajectory.last.relative.lat;
    }
    set lat(newLat) {
        let relative = this.trajectory.last.relative;
        relative.lat = newLat;
        this.trajectory.last.relative = relative;
    }
    get lon() {
        return this.trajectory.last.relative.lon;
    }
    set lon(newLon) {
        let relative = this.trajectory.last.relative;
        relative.lon = newLon;
        this.trajectory.last.relative = relative;
    }
    get vx() {
        return (this.trajectory.last.relative.x - this.trajectory.nexto.relative.x) / this.dt;
    }
    get vy() {
        return (this.trajectory.last.relative.y - this.trajectory.nexto.relative.y) / this.dt;
    }
    get vz() {
        return (this.trajectory.last.relative.z - this.trajectory.nexto.relative.z) / this.dt;
    }
    get vr() {
        return (this.trajectory.last.relative.r - this.trajectory.nexto.relative.r) / this.dt;
    }
    get vrxy() {
        return (this.trajectory.last.relative.rxy - this.trajectory.nexto.relative.rxy) / this.dt;
    }
    get vtheta() {
        return (this.trajectory.last.relative.theta - this.trajectory.nexto.relative.theta) / this.dt;
    }
    get vphi() {
        return (this.trajectory.last.relative.phi - this.trajectory.nexto.relative.phi) / this.dt;
    }
    init(u0 = Vector3_1.Vector3.zeros, v0 = Vector3_1.Vector3.zeros, f = this.solver.f, dt0 = this.solver.dt0) {
        this.solver.f = f;
        this.solver.dt0 = dt0;
        this.trajectory.nexto.relative = v0.mulc(-dt0).add(u0);
        this.trajectory.last.relative = u0;
        this.trajectory.dt[this.trajectory.lastStepIndex] = dt0;
        return this;
    }
    update(dt, origin) {
        this.solver.buffer(this.trajectory, dt, origin);
        return this;
    }
    reframe(p) {
        this.trajectory.origin = p.trajectory.absolute;
        return this;
    }
    copy() {
        return new Point(this.trajectory.copy(), this.solver, this.mass);
    }
    static zeros(size = 2, frame = Vector3_1.Vector3.zeros, mass = 0) {
        return new Point(BufferTrajectory_1.BufferTrajectory.zeros(frame, size), new Solver_1.Solver(), mass);
    }
}
exports.Point = Point;
//# sourceMappingURL=Point.js.map