"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Solver_1 = require("./Solver");
const Vector3_1 = require("./Vector3");
exports.defaultOptions = { copy: true, size: 2 };
class PointSet {
    constructor(points, solver = new Solver_1.Solver(), frame = points[0]) {
        this.points = points;
        this._frame = frame;
        this.points.forEach(point => point.solver = solver);
        this.field = solver.field;
    }
    get options() {
        return { copy: this.copy, size: this.points[0].options.size };
    }
    set options(newOptions) {
        const size = newOptions.size || this.options.size;
        this.copy = newOptions.copy || this.copy;
        this.points.forEach(point => point.trajectory.size = size);
    }
    get frame() {
        return this._frame;
    }
    set frame(newFrame) {
        this._frame = newFrame;
        this.points.forEach(point => point.reframe(newFrame));
    }
    get solver() {
        return this.points[0].solver;
    }
    set solver(newSolver) {
        this.points.forEach(point => point.solver = newSolver);
        this.field = newSolver.field;
    }
    get method() {
        return this.points[0].solver.method;
    }
    set method(newMethod) {
        this.points.forEach(point => point.solver.method = newMethod);
    }
    get dt() {
        return this.points[0].dt;
    }
    set dt(newDt) {
        this.points.forEach(point => {
            point.solver.dt1 = newDt;
            point.dt = newDt;
        });
    }
    get field() {
        return this.points[0].solver.field;
    }
    set field(newField) {
        this.points.forEach(point => point.field = newField);
    }
    get center() {
        return this.points.reduce((acc, point) => acc.add(point.position), Vector3_1.Vector3.zeros).div(this.points.length);
    }
    set center(newCenter) {
        const center = this.center;
        this.points.forEach(point => point.position = point.position.sub(center).add(newCenter));
    }
    get barycenter() {
        const mass = this.points.reduce((acc, point) => acc + point.mass, 0) || this.points.length;
        return this.points.reduce((acc, point) => acc.add(point.position.mul(point.mass || 0)), Vector3_1.Vector3.zeros).div(mass);
    }
    set barycenter(newCenter) {
        const center = this.barycenter;
        this.points.forEach(point => point.position = point.position.sub(center).add(newCenter));
    }
    update(dt, origin) {
        this.points = this.points.map((point) => point.copy().update(dt, origin));
        return this;
    }
    reframe(p) {
        this.frame = p;
        return this;
    }
}
exports.PointSet = PointSet;
//# sourceMappingURL=PointSet.js.map