"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Vector3_1 = require("./Vector3");
const Pair3_1 = require("./Pair3");
const Trajectory_1 = require("./Trajectory");
const compute = [
    (f, u1, u0, t, dt0, dt1 = dt0) => u1.mulc(2).sub(u0).add(f(u1, t).mul(dt0 * dt1))
];
var methods;
(function (methods) {
    methods[methods["EULER"] = 0] = "EULER";
})(methods = exports.methods || (exports.methods = {}));
class Solver {
    constructor(f = () => Vector3_1.Vector3.zeros, dt = 1, method = methods.EULER) {
        this.f = f;
        this.dt0 = dt;
        this.dt1 = dt;
        this.method = method;
    }
    step(u1, u0, t = 0, dt = this.dt1) {
        this.dt0 = this.dt1;
        this.dt1 = dt;
        return compute[this.method](this.f, u1, u0, t, this.dt0, this.dt1);
    }
    buffer(trajectory, dt = this.dt1, origin = trajectory.pairs[trajectory.lastIndex].origin) {
        let next = this.step(trajectory.last.position, trajectory.nexto.position, trajectory.duration(), dt);
        trajectory.add(new Pair3_1.Pair3(origin, next), this.dt0);
        return trajectory;
    }
    initialTransform(u0, v0, dt0 = this.dt0, dt1 = this.dt0) {
        this.dt0 = dt0;
        this.dt1 = dt1;
        let u1 = v0.mulc(dt0).add(u0);
        return u1.add(this.f(u1, 0).mul(dt0 * dt1 / 2));
    }
    solve(u0, v0, count, dt = this.dt1) {
        let u = new Array(count);
        this.dt0 = dt;
        this.dt1 = dt;
        u[0] = u0.copy();
        u[1] = this.initialTransform(u0, v0, this.dt0);
        for (let i = 2; i < count; i++) {
            u[i] = this.step(u[i - 1], u[i - 2], i * dt, dt);
        }
        return u;
    }
    solveMax(u0, v0, tmax, dt = this.dt1) {
        return this.solve(u0, v0, Math.floor(tmax / dt), dt);
    }
    trajectory(u0, v0, count, dt = this.dt1, origin = Vector3_1.Vector3.zeros) {
        return Trajectory_1.Trajectory.discrete(this.solve(u0, v0, count, dt), dt, origin);
    }
    trajectoryMax(u0, v0, tmax, dt = this.dt1, origin = Vector3_1.Vector3.zeros) {
        return Trajectory_1.Trajectory.discrete(this.solveMax(u0, v0, tmax, dt), dt, origin);
    }
}
exports.Solver = Solver;
//# sourceMappingURL=Solver.js.map