"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
class Vector3 {
    constructor(x = 0, y = 0, z = 0) {
        this.xyz = [x, y, z];
    }
    get r() {
        return Math.sqrt(this.scal(this));
    }
    set r(newR) {
        this.rthph = [newR, this.theta, this.phi];
    }
    get theta() {
        return Math.atan2(this.y, this.x);
    }
    set theta(newTheta) {
        this.rthph = [this.r, newTheta, this.phi];
    }
    get phi() {
        return Math.atan2(this.rxy, this.z);
    }
    set phi(newPhi) {
        this.rthph = [this.r, this.theta, newPhi];
    }
    get rxy() {
        return Math.sqrt(this.x * this.x + this.y * this.y);
    }
    set rxy(newRxy) {
        this.rthz = [newRxy, this.theta, this.z];
    }
    get lat() {
        return Math.PI / 2 - this.phi;
    }
    set lat(newLat) {
        this.rthph = [this.r, this.theta, Math.PI / 2 - newLat];
    }
    get lon() {
        return this.theta <= Math.PI ? this.theta : this.theta - 2 * Math.PI;
    }
    set lon(newLat) {
        this.rthph = [this.r, newLat >= 0 ? newLat : newLat + 2 * Math.PI, this.phi];
    }
    get xyz() {
        return [this.x, this.y, this.z];
    }
    set xyz(coordinates) {
        this.x = coordinates[0];
        this.y = coordinates[1];
        this.z = coordinates[2];
    }
    get rthz() {
        return [this.rxy, this.theta, this.z];
    }
    set rthz(coordinates) {
        this.x = coordinates[0] * Math.cos(coordinates[1]);
        this.y = coordinates[0] * Math.sin(coordinates[1]);
        this.z = coordinates[2];
    }
    get rthph() {
        return [this.r, this.theta, this.phi];
    }
    set rthph(coordinates) {
        this.x = coordinates[0] * Math.sin(coordinates[2]) * Math.cos(coordinates[1]);
        this.y = coordinates[0] * Math.sin(coordinates[2]) * Math.sin(coordinates[1]);
        this.z = coordinates[0] * Math.cos(coordinates[2]);
    }
    copy() {
        return new Vector3(this.x, this.y, this.z);
    }
    fill(s) {
        this.x = s;
        this.y = s;
        this.z = s;
        return this;
    }
    fillc(s) {
        return this.copy().fill(s);
    }
    add(u) {
        this.x += u.x;
        this.y += u.y;
        this.z += u.z;
        return this;
    }
    addc(u) {
        return this.copy().add(u);
    }
    sub(u) {
        this.x -= u.x;
        this.y -= u.y;
        this.z -= u.z;
        return this;
    }
    subc(u) {
        return this.copy().sub(u);
    }
    opp() {
        this.x *= -1;
        this.y *= -1;
        this.z *= -1;
        return this;
    }
    oppc() {
        return this.copy().opp();
    }
    mul(s) {
        this.x *= s;
        this.y *= s;
        this.z *= s;
        return this;
    }
    mulc(s) {
        return this.copy().mul(s);
    }
    div(s) {
        this.x /= s;
        this.y /= s;
        this.z /= s;
        return this;
    }
    divc(s) {
        return this.copy().div(s);
    }
    scal(u) {
        return this.x * u.x + this.y * u.y + this.z * u.z;
    }
    cross(u) {
        this.xyz = [
            this.y * u.z - this.z * u.y,
            this.z * u.x - this.x * u.z,
            this.x * u.y - this.y * u.x
        ];
        return this;
    }
    crossc(u) {
        return this.copy().cross(u);
    }
    trans() {
        return this;
    }
    transc() {
        return this.copy();
    }
    prod(u) {
        return this.cross(u);
    }
    prodc(u) {
        return this.crossc(u);
    }
    dist(u) {
        return this.copy().sub(u).r;
    }
    angle(u) {
        return Math.atan(this.tan(u));
    }
    cos(u) {
        return (!this.isZero() && !u.isZero()) ? this.scal(u) / (this.r * u.r) : 1;
    }
    sin(u) {
        return (!this.isZero() && !u.isZero()) ? this.copy().cross(u).r / (this.r * u.r) : 0;
    }
    tan(u) {
        return this.sin(u) / this.cos(u);
    }
    isEqual(u) {
        return Math.abs(this.dist(u)) < Number.EPSILON;
    }
    isZero() {
        return this.r < Number.EPSILON;
    }
    toString() {
        return `(${this.x.toExponential(2)} ${this.y.toExponential(2)} ${this.z.toExponential(2)})`;
    }
    to1D() {
        return this.xyz;
    }
    static get zeros() {
        return new Vector3();
    }
    static get ones() {
        return Vector3.scal(1);
    }
    static scal(s) {
        return new Vector3(s, s, s);
    }
    static get ex() {
        return Vector3.e(0);
    }
    static get ey() {
        return Vector3.e(1);
    }
    static get ez() {
        return Vector3.e(2);
    }
    static e(k) {
        return new Vector3(k === 0 ? 1 : 0, k === 1 ? 1 : 0, k === 2 ? 1 : 0);
    }
    static er(u) {
        return new Vector3(Math.sin(u.phi) * Math.cos(u.theta), Math.sin(u.phi) * Math.sin(u.theta), Math.cos(u.phi));
    }
    static etheta(u) {
        return new Vector3(-Math.sin(u.theta), Math.cos(u.theta), 0);
    }
    static ephi(u) {
        return new Vector3(Math.cos(u.phi) * Math.cos(u.theta), Math.cos(u.phi) * Math.sin(u.theta), -Math.sin(u.phi));
    }
    static derivative(vectors, dt = 1) {
        const steps = (typeof dt == "number") ? Array(vectors.length).fill(dt) : dt;
        const der = new Array(vectors.length - 1);
        for (let i = 0; i < vectors.length - 1; i++) {
            der[i] = vectors[i + 1].copy().sub(vectors[i]).div(steps[i]);
        }
        return der;
    }
    static from1D(arr) {
        return new Vector3(arr[0], arr[1], arr[2]);
    }
    static cylindrical(rxy, theta, z) {
        const u = new Vector3();
        u.rthz = [rxy, theta, z];
        return u;
    }
    static spherical(r, theta, phi) {
        const u = new Vector3();
        u.rthph = [r, theta, phi];
        return u;
    }
}
exports.Vector3 = Vector3;
//# sourceMappingURL=Vector3.js.map